@@ -66,7 +66,7 @@ PHP_INI_BEGIN()
 			decode_binary, zend_yaml_globals, yaml_globals)
 	STD_PHP_INI_ENTRY("yaml.decode_timestamp", "0", PHP_INI_ALL, OnUpdateLong,
 			decode_timestamp, zend_yaml_globals, yaml_globals)
-	STD_PHP_INI_ENTRY("yaml.decode_php", "1", PHP_INI_ALL, OnUpdateBool,
+	STD_PHP_INI_ENTRY("yaml.decode_php", "0", PHP_INI_ALL, OnUpdateBool,
 			decode_php, zend_yaml_globals, yaml_globals)
 	STD_PHP_INI_ENTRY("yaml.output_canonical", "0", PHP_INI_ALL, OnUpdateBool,
 			output_canonical, zend_yaml_globals, yaml_globals)
@@ -267,7 +267,7 @@ static PHP_GINIT_FUNCTION(yaml)
 {
 	yaml_globals->decode_binary = 0;
 	yaml_globals->decode_timestamp = 0;
-	yaml_globals->decode_php = 1;
+	yaml_globals->decode_php = 0;
 	yaml_globals->timestamp_decoder = NULL;
 	yaml_globals->output_canonical = 0;
 	yaml_globals->output_indent = 2;
@@ -281,50 +281,36 @@ static PHP_GINIT_FUNCTION(yaml)
  */
 static int php_yaml_check_callbacks(HashTable *callbacks TSRMLS_DC)
 {
-	zval **entry = { 0 };
-	char *key = { 0 };
-	uint key_len = 0;
-	ulong idx = 0L;
+	zval *entry;
+	zend_string *key;
 
-	zend_hash_internal_pointer_reset(callbacks);
+	ZEND_HASH_FOREACH_STR_KEY_VAL(callbacks, key, entry) {
+		if (key) {
+			zend_string *name;
 
-	while (SUCCESS ==
-			zend_hash_get_current_data(callbacks, (void **) &entry)) {
-		int key_type = zend_hash_get_current_key_ex(
-				callbacks, &key, &key_len, &idx, 0, NULL);
-
-		if (key_type == HASH_KEY_IS_STRING) {
-			char *name;
-
-			if (!ZEND_IS_CALLABLE(*entry, 0, &name)) {
+			if (!ZEND_IS_CALLABLE(entry, 0, &name)) {
 				if (name != NULL) {
 					php_error_docref(NULL TSRMLS_CC, E_WARNING,
 							"Callback for tag '%s', '%s' is not valid",
-							key, name);
+							key->val, name->val);
 					efree(name);
 
 				} else {
 					php_error_docref(NULL TSRMLS_CC, E_WARNING,
-							"Callback for tag '%s' is not valid", key);
+							"Callback for tag '%s' is not valid", key->val);
 				}
 				return FAILURE;
 			}
 
-			if (!strcmp(key, YAML_TIMESTAMP_TAG)) {
-				YAML_G(timestamp_decoder) = *entry;
-			}
-
-			if (name != NULL) {
-				efree(name);
+			if (!memcmp(key->val, YAML_TIMESTAMP_TAG, sizeof(YAML_TIMESTAMP_TAG))) {
+				YAML_G(timestamp_decoder) = entry;
 			}
 
 		} else {
 			php_error_docref(NULL TSRMLS_CC, E_NOTICE,
 					"Callback key should be a string");
 		}
-
-		zend_hash_move_forward(callbacks);
-	}
+	} ZEND_HASH_FOREACH_END();
 
 	return SUCCESS;
 }
@@ -335,25 +321,23 @@ static int php_yaml_check_callbacks(HashTable *callbacks TSRMLS_DC)
  Takes a YAML encoded string and converts it to a PHP variable. */
 PHP_FUNCTION(yaml_parse)
 {
-	char *input = { 0 };
-	int input_len = 0;
-	long pos = 0;
+	zend_string *input;
+	zend_long pos = 0;
 	zval *zndocs = { 0 };
 	zval *zcallbacks = { 0 };
 
 	parser_state_t state;
-	zval *yaml = { 0 };
-	long ndocs = 0;
+	zval yaml;
+	zend_long ndocs = 0;
 
 	memset(&state, 0, sizeof(state));
 	state.have_event = 0;
-	state.aliases = NULL;
 	state.callbacks = NULL;
 
 	YAML_G(timestamp_decoder) = NULL;
 
 	if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC,
-					"s|lza/", &input, &input_len, &pos, &zndocs,
+					"S|lz/a/", &input, &pos, &zndocs,
 					&zcallbacks)) {
 		return;
 	}
@@ -371,30 +355,28 @@ PHP_FUNCTION(yaml_parse)
 	}
 
 	yaml_parser_initialize(&state.parser);
-	yaml_parser_set_input_string(
-			&state.parser, (unsigned char *) input, (size_t) input_len);
+	yaml_parser_set_input_string(&state.parser, (unsigned char *)input->val, input->len);
 
 
 	if (pos < 0) {
-		yaml = php_yaml_read_all(&state, &ndocs TSRMLS_CC);
-
+		php_yaml_read_all(&state, &ndocs, &yaml TSRMLS_CC);
 	} else {
-		yaml = php_yaml_read_partial(&state, pos, &ndocs TSRMLS_CC);
+		php_yaml_read_partial(&state, pos, &ndocs, &yaml TSRMLS_CC);
 	}
 
 	yaml_parser_delete(&state.parser);
 
 	if (zndocs != NULL) {
 		/* copy document count to var user sent in */
-		zval_dtor(zndocs);
+		zval_ptr_dtor(zndocs);
 		ZVAL_LONG(zndocs, ndocs);
 	}
 
-	if (NULL == yaml) {
+	if (Z_TYPE_P(&yaml) == IS_UNDEF) {
 		RETURN_FALSE;
 	}
 
-	RETURN_ZVAL(yaml, 1, 1);
+	RETURN_ZVAL(&yaml, 1, 1);
 }
 /* }}} yaml_parse */
 
@@ -404,8 +386,8 @@ PHP_FUNCTION(yaml_parse)
 PHP_FUNCTION(yaml_parse_file)
 {
 	char *filename = { 0 };
-	int filename_len = 0;
-	long pos = 0;
+	size_t filename_len = 0;
+	zend_long pos = 0;
 	zval *zndocs = { 0 };
 	zval *zcallbacks = { 0 };
 
@@ -413,12 +395,11 @@ PHP_FUNCTION(yaml_parse_file)
 	FILE *fp = { 0 };
 
 	parser_state_t state;
-	zval *yaml = { 0 };
-	long ndocs = 0;
+	zval yaml;
+	zend_long ndocs = 0;
 
 	memset(&state, 0, sizeof(state));
 	state.have_event = 0;
-	state.aliases = NULL;
 	state.callbacks = NULL;
 
 	YAML_G(timestamp_decoder) = NULL;
@@ -442,7 +423,7 @@ PHP_FUNCTION(yaml_parse_file)
 	}
 
 	if (NULL == (stream = php_stream_open_wrapper(filename, "rb",
-			IGNORE_URL | ENFORCE_SAFE_MODE | REPORT_ERRORS | STREAM_WILL_CAST,
+			IGNORE_URL | REPORT_ERRORS | STREAM_WILL_CAST,
 			NULL))) {
 		RETURN_FALSE;
 	}
@@ -457,10 +438,10 @@ PHP_FUNCTION(yaml_parse_file)
 	yaml_parser_set_input_file(&state.parser, fp);
 
 	if (pos < 0) {
-		yaml = php_yaml_read_all(&state, &ndocs TSRMLS_CC);
+		php_yaml_read_all(&state, &ndocs, &yaml TSRMLS_CC);
 
 	} else {
-		yaml = php_yaml_read_partial(&state, pos, &ndocs TSRMLS_CC);
+		php_yaml_read_partial(&state, pos, &ndocs, &yaml TSRMLS_CC);
 	}
 
 	yaml_parser_delete(&state.parser);
@@ -472,11 +453,11 @@ PHP_FUNCTION(yaml_parse_file)
 		ZVAL_LONG(zndocs, ndocs);
 	}
 
-	if (NULL == yaml) {
+	if (Z_TYPE_P(&yaml) == IS_UNDEF) {
 		RETURN_FALSE;
 	}
 
-	RETURN_ZVAL(yaml, 1, 1);
+	RETURN_ZVAL(&yaml, 1, 1);
 }
 /* }}} yaml_parse_file */
 
@@ -486,22 +467,21 @@ PHP_FUNCTION(yaml_parse_file)
 PHP_FUNCTION(yaml_parse_url)
 {
 	char *url = { 0 };
-	int url_len = 0;
-	long pos = 0;
+	size_t url_len = 0;
+	zend_long pos = 0;
 	zval *zndocs = { 0 };
 	zval *zcallbacks = { 0 };
 
 	php_stream *stream = { 0 };
-	char *input = { 0 };
+	zend_string *input;
 	size_t size = 0;
 
 	parser_state_t state;
-	zval *yaml = { 0 };
-	long ndocs = 0;
+	zval yaml;
+	zend_long ndocs = 0;
 
 	memset(&state, 0, sizeof(state));
 	state.have_event = 0;
-	state.aliases = NULL;
 	state.callbacks = NULL;
 
 	YAML_G(timestamp_decoder) = NULL;
@@ -523,26 +503,27 @@ PHP_FUNCTION(yaml_parse_url)
 		state.eval_func = eval_scalar;
 	}
 
-	if (NULL == (stream = php_stream_open_wrapper(url, "rb",
-					ENFORCE_SAFE_MODE | REPORT_ERRORS, NULL))) {
+	if (NULL == (stream = php_stream_open_wrapper(url, "rb", REPORT_ERRORS, NULL))) {
 		RETURN_FALSE;
 	}
 
-	size = php_stream_copy_to_mem(stream, &input, PHP_STREAM_COPY_ALL, 0);
+	input = php_stream_copy_to_mem(stream, PHP_STREAM_COPY_ALL, 0);
+	php_stream_close(stream);
+	if (input == NULL) {
+		RETURN_FALSE;
+	}
 
 	yaml_parser_initialize(&state.parser);
 	yaml_parser_set_input_string(&state.parser, (unsigned char *)input, size);
 
 	if (pos < 0) {
-		yaml = php_yaml_read_all(&state, &ndocs TSRMLS_CC);
-
+		php_yaml_read_all(&state, &ndocs, &yaml TSRMLS_CC);
 	} else {
-		yaml = php_yaml_read_partial(&state, pos, &ndocs TSRMLS_CC);
+		php_yaml_read_partial(&state, pos, &ndocs, &yaml TSRMLS_CC);
 	}
 
 	yaml_parser_delete(&state.parser);
-	php_stream_close(stream);
-	efree(input);
+	zend_string_release(input);
 
 	if (zndocs != NULL) {
 		/* copy document count to var user sent in */
@@ -550,11 +531,11 @@ PHP_FUNCTION(yaml_parse_url)
 		ZVAL_LONG(zndocs, ndocs);
 	}
 
-	if (NULL == yaml) {
+	if (Z_TYPE_P(&yaml) == IS_UNDEF) {
 		RETURN_FALSE;
 	}
 
-	RETURN_ZVAL(yaml, 1, 1);
+	RETURN_ZVAL(&yaml, 1, 1);
 }
 /* }}} yaml_parse_url */
 
@@ -564,13 +545,13 @@ PHP_FUNCTION(yaml_parse_url)
 PHP_FUNCTION(yaml_emit)
 {
 	zval *data = { 0 };
-	long encoding = YAML_ANY_ENCODING;
-	long linebreak = YAML_ANY_BREAK;
+	zend_long encoding = YAML_ANY_ENCODING;
+	zend_long linebreak = YAML_ANY_BREAK;
 	zval *zcallbacks = { 0 };
 	HashTable *callbacks = { 0 };
 
 	yaml_emitter_t emitter = { 0 };
-	smart_str str = { 0 };
+	smart_string str = { 0 };
 
 	if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS()TSRMLS_CC, "z/|lla/",
 				&data, &encoding, &linebreak, &zcallbacks)) {
@@ -597,14 +578,14 @@ PHP_FUNCTION(yaml_emit)
 	if (SUCCESS == php_yaml_write_impl(
 				&emitter, data, (yaml_encoding_t) encoding,
 				callbacks TSRMLS_CC)) {
-		RETVAL_STRINGL(str.c, str.len, 1);
+		RETVAL_STRINGL(str.c, str.len);
 
 	} else {
 		RETVAL_FALSE;
 	}
 
 	yaml_emitter_delete(&emitter);
-	smart_str_free(&str);
+	smart_string_free(&str);
 }
 /* }}} yaml_emit */
 
@@ -613,21 +594,19 @@ PHP_FUNCTION(yaml_emit)
    */
 PHP_FUNCTION(yaml_emit_file)
 {
-	char *filename = { 0 };
-	int filename_len = 0;
+	zend_string *filename = { 0 };
 	php_stream *stream = { 0 };
 	FILE *fp = { 0 };
 	zval *data = { 0 };
-	long encoding = YAML_ANY_ENCODING;
-	long linebreak = YAML_ANY_BREAK;
+	zend_long encoding = YAML_ANY_ENCODING;
+	zend_long linebreak = YAML_ANY_BREAK;
 	zval *zcallbacks = { 0 };
 	HashTable *callbacks = { 0 };
 
 	yaml_emitter_t emitter = { 0 };
 
-	if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "sz/|lla/",
-			&filename, &filename_len,
-			&data, &encoding, &linebreak, &zcallbacks)) {
+	if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "Sz/|lla/",
+			&filename, &data, &encoding, &linebreak, &zcallbacks)) {
 		return;
 	}
 
@@ -638,8 +617,8 @@ PHP_FUNCTION(yaml_emit_file)
 		}
 	}
 
-	if (NULL == (stream = php_stream_open_wrapper(filename, "wb",
-			IGNORE_URL | ENFORCE_SAFE_MODE | REPORT_ERRORS | STREAM_WILL_CAST,
+	if (NULL == (stream = php_stream_open_wrapper(filename->val, "wb",
+			IGNORE_URL | REPORT_ERRORS | STREAM_WILL_CAST,
 			NULL))) {
 		RETURN_FALSE;
 	}
